/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2008 Adrien Bustany <madcat@mymadcat.com>
 * Copyright (c) 2008 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _NEPOMUK_ANNOTATION_PLUGIN_H_
#define _NEPOMUK_ANNOTATION_PLUGIN_H_

#include <QtCore/QObject>

#include <nepomuk/nepomuk_export.h>
#include <nepomuk/resource.h>

#include <Nepomuk/Types/Property>

#include "annotationproperty.h"

#include <KPluginFactory>
#include <KPluginLoader>

namespace Soprano {
	class Node;
}

namespace Nepomuk {

    class AnnotationResource;
    class AnnotationResult;
    class Annotation;

    /**
     * \class AnnotationPlugin annotationplugin.h AnnotationPlugin
     *
     * \brief A generic implementation of a Nepomuk AnnotationPlugin
     *
     * AnnotationPlugins are designed to provide an easy to use interface to data stored into Nepomuk.
     * The interface is all asynchronous, if you ever need to use AnnotationPlugins in a threaded environment,
     * use the SyncAnnotationPlugin class.
     *
     * All the get* functions return an AnnotationResult object.
     *
     * \sa SyncAnnotationPlugin
     * \sa AnnotationResult
     *
     * \author Adrien Bustany <madcat@mymadcat.com>, Sebastian Trueg <trueg@kde.org>
     */
    class NEPOMUK_EXPORT AnnotationPlugin : public QObject
    {
        Q_OBJECT

    public:
        AnnotationPlugin(QObject* parent);
        virtual ~AnnotationPlugin();

        /**
         * Returns true if the plugin is ready, false else.
         * This function is useful when you get several times an instance of the plugin. As the factory will only create one
         * instance of each plugin, the ready() signal will only be emitted once.
         * If you execute commands on a non-ready plugin, they'll be queued and executed when the plugin is ready.
         */
        virtual bool isReady() const;

        /**
         * Returns the name of the plugin
         */
        virtual QString name() const;

        /**
         * Returns an icon name usable for example with KIcon
         */
        virtual QString iconName() const;

    Q_SIGNALS:
        /**
         * Emitted when addRelation finishes
         */
        void finished();

        /**
         * Emitted when the plugin is ready to use. If any command is issued before the
         * plugin is ready, it'll be queued and executed automatically when possible.
         */
        // trueg: I think this is overkill. Just let everyone use their own instance of the
        //        plugin
        void ready();

/** \cond hide_private_signals_from_moc */
#ifndef Q_MOC_RUN
    private: // don't tell moc, but this signal is in fact private
#endif
/** \endcond */
        /**
         * Emitted once the plugin has listed all possible annotations.
         *
         * This is a private signal, it can't be emitted directly 
         * by subclasses of AnnotationPlugin, use emitFinished() instead.
         *
         * \param plugin The plugin emitting the signal.
         */
        void finished( Nepomuk::AnnotationPlugin* plugin );

        /**
         * Emitted once a new possible Annotation has been found.
         * Until finished has been emitted, more annotations can still arrive.
         *
         * This is a private signal, it can't be emitted directly 
         * by subclasses of AnnotationPlugin, use addNewAnnotation() instead.
         *
         * \param annotation The new Annotation object. While the object is destroyed
         * with the plugin, it is recommended to do the destruction manually once
         * the Annotation object is no longer needed.
         */
        void newAnnotation( Nepomuk::Annotation* annotation );

    public Q_SLOTS:
        /**
         * Searches for possible annotations of resource that match filter.
         *
         * Annotations are listed via newAnnotation. Once all have been listed,
         * finished is emitted.
         *
         * \param resource The resource to be annotated
         * \param filter An optional string filter (most likely input from the user)
         *
         * FIXME: What about annotating multiple resources at the same time?
         */
        void getPossibleAnnotations( const Nepomuk::Resource& resource, const QString& filter = QString() );

        /**
         * Retrieves all the resources related with the \a filter keyword
         */
        virtual AnnotationResult* getResources(const QString& filter);

        /**
         * Retrieves all the resources related with the \a filter keyword, and having the given \a property
         */
        virtual AnnotationResult* getResources(const QString& filter, const Types::Property& property, const Soprano::Node& object);

        /**
         * Retrieves the resource with the given \a uri
         */
        virtual AnnotationResult* getResource(const QUrl& resourceUri);

        /**
         * Retrieves all the resources linked to \p resourceUri . This function uses nao:isRelated. This relation being a
         * symmetric property, we also do a reverse query.
         */
        virtual AnnotationResult* getResourcesRelatedTo(const QUrl& resourceUri);

        /**
         * Retrieves all the relevant relations for the given mimetype as a list of <Property, Relevance> tuples
         */
        virtual QList<AnnotationProperty> getProperties(const QString& mimetype);

        /**
         * Retrieves all the relevant relations for the given RDF type as a list of <Property, Relevance> tuples
         */
        virtual QList<AnnotationProperty> getProperties(const QUrl& rdfType);

        /**
         * Effectively associates the file at the given url with the relation identified by the uri
         * trueg: why no result here?
         */
        virtual void addRelation(const Types::Property& property, const QUrl& subject, const Soprano::Node& object);
        virtual void addRelation(const AnnotationProperty& property, const QUrl& subject, const Soprano::Node& object);

    protected:
        /**
         * Creates a new AnnotationResult with the given \p parent
         *
         * This is used by subclasses to create a new AnnotationResult objects, since the constructor
         * is private.
         */
        AnnotationResult* createResult(AnnotationPlugin* parent);

        /**
         * Converts a soprano node to a QString usable in a sparql query
         */
        QString nodeToSparql(const Soprano::Node& node);

        /**
         * Only for internal use : used to queue commands when the plugin is not ready
         * Note : getProperties cannot be enqueud since it does not return void.
         */
        enum CommandName {
            command_getResources,
            command_getResourcesFiltered,
            command_getResource,
            command_addRelationNepomukProperty,
            command_addRelationAnnotationProperty
        };
        typedef QPair<AnnotationPlugin::CommandName, QHash<QString, QVariant> > Command;

        QMap<AnnotationResult*, AnnotationPlugin::Command> m_queuedCommands;

        /**
         * Used by the plugin to memorize the commands to execute once it'll be ready
         */
        void enqueueCommand(AnnotationResult* r, const AnnotationPlugin::Command &c);

        /**
         * Used by the plugin to execute all the enqueued commands once it's ready
         */
        void executeQueuedCommands();

        /**
         * Subclasses should reimplement this to provide a list of Annotation objects.
         * Be aware that the method may be called again before the finished signal is
         * emitted. In that case the plugin should restart looking for annotations 
         * without emitting the finished signal.
         *
         * \param resource The resource to be annotated
         * \param filter An optional string filter (most likely input from the user)s
         */
        virtual void doGetPossibleAnnotations( const Nepomuk::Resource& resource, const QString& filter );

    protected Q_SLOTS:
        /**
         * Emits the finished signal.
         */
        void emitFinished();

        /**
         * Subclasses need to call this for each new Annotation as a result of
         * getPossibleAnnotations.
         *
         * This will emit the newAnnotation signal.
         */
        void addNewAnnotation( Nepomuk::Annotation* );

    private:
        class Private;
        Private* const d;
    };

} // End namespace Nepomuk


/**
 * Export a %Nepomuk Annotation Plugin.
 *
 * \param classname The name of the Nepomuk::AnnotationPlugin subclass to export.
 * \param libname The name of the library which should export the plugin.
 */
#define NEPOMUK_EXPORT_ANNOTATION_PLUGIN( classname, libname )  \
K_PLUGIN_FACTORY(factory, registerPlugin<classname>();)     \
K_EXPORT_PLUGIN(factory(#libname))

#endif
