/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2008 Adrien Bustany <madcat@mymadcat.com>
 * Copyright (C) 2008 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef ANNOTATIONPLUGINFACTORY_H
#define ANNOTATIONPLUGINFACTORY_H

#include <QtCore/QObject>
#include <QtCore/QString>
#include <QtCore/QHash>
#include <QtCore/QUrl>
#include <nepomuk/nepomuk_export.h>
#include <KSharedPtr>

class KService;

namespace Nepomuk {

    class AnnotationPlugin;

    namespace Types {
        class Property;
        class Class;
    }

    /**
     * \class AnnotationPluginFactory annotationpluginfactory.h AnnotationPluginFactory
     *
     * \brief Factory to create AnnotationPlugins based on the property you want to use. It can also return all known AnnotationPlugins.
     *
     * Once created, plugins are cached to reduce memory consumption. This class is a singleton, you should use the instance() function to
     * get a pointer to it. It will be created when needed, and be destroyed with the application. All created plugins are parented to the
     * factory, hence you don't need to worry about deleting them. You can delete them if you want though.
     *
     * \author Adrien Bustany <madcat@mymadcat.com>, Sebastian Trueg <trueg@kde.org>
     */
    class NEPOMUK_EXPORT AnnotationPluginFactory : public QObject
    {
        Q_OBJECT

    public:
        /**
         * Create a new instance of AnnotationPluginFactory. It will
         * maintain a list of annotation plugins.
         */
        AnnotationPluginFactory(QObject* parent = 0);
        ~AnnotationPluginFactory();

        /**
         * \deprecated Create your own instance
         */
        static AnnotationPluginFactory* instance();

        /**
         * Returns a list with all the AnnotationPlugins than can handle the given \a property.
         * All the plugins have the factory as a parent, so they'll be deleted when the factory dies
         * (that is, when the application quits, since the factory is a singleton with qApp as a parent)
         *
         * Be aware that one instance of AnnotationPluginFactory will return the same plugin instances
         * on subsequent calls.
         */
        QList<AnnotationPlugin*> getPluginsForProperty(const Nepomuk::Types::Property& property);

        /**
         * Returns a list with all the AnnotationPlugins that return the given \a type
         * All the plugins have the factory as a parent, so they'll be deleted when the factory dies
         * (that is, when the application quits, since the factory is a singleton with qApp as a parent)
         *
         * Be aware that one instance of AnnotationPluginFactory will return the same plugin instances
         * on subsequent calls.
         */
        QList<AnnotationPlugin*> getPluginsForType(const QUrl& rdfType);

        /**
         * \return a list of plugins that can create Annotation objects for resource of type \p rdfType
         *
         * Be aware that one instance of AnnotationPluginFactory will return the same plugin instances
         * on subsequent calls.
         */
        QList<Nepomuk::AnnotationPlugin*> getPluginsSupportingAnnotationOfType( const Types::Class& rdfType );

        /**
         * \return a list of plugins that can create Annotation objects for resource of any type in \p rdfTypes
         *
         * Be aware that one instance of AnnotationPluginFactory will return the same plugin instances
         * on subsequent calls.
         */
        QList<Nepomuk::AnnotationPlugin*> getPluginsSupportingAnnotationOfTypes( const QList<Types::Class>& rdfTypes );

        /**
         * \return a list of plugins that can create Annotation objects for \p resource
         *
         * Be aware that one instance of AnnotationPluginFactory will return the same plugin instances
         * on subsequent calls.
         */
        QList<Nepomuk::AnnotationPlugin*> getPluginsSupportingAnnotationForResource( const QUrl& resource );

        /**
         * Tries to get the plugin with the given library \p name (usually something like nepomuk_xxxanotationplugin).
         * Can return NULL if ne plugin is found.
         *
         * Be aware that one instance of AnnotationPluginFactory will return the same plugin instances
         * on subsequent calls.
         */
        AnnotationPlugin* getPluginByLibraryName(const QString& name);

        /**
         * Returns a list with all the known AnnotationPlugins.
         * All the plugins have the factory as a parent, so they'll be deleted when the factory dies
         * (that is, when the application quits, since the factory is a singleton with qApp as a parent)
         *
         * Be aware that one instance of AnnotationPluginFactory will return the same plugin instances
         * on subsequent calls.
         */
        QList<AnnotationPlugin*> getAllPlugins();

    private:
        class Private;
        Private* const d;

        /**
         * Returns an instance of the plugin described by the given service. If a cached instance exists in memory, then
         * it's returned. Else, a new one is created. Caching is done on the library name since one single library can
         * handle multiple relations.<br>
         * All the plugins have the factory as a parent, so they'll be deleted when the factory dies
         * (that is, when the application quits, since the factory is a singleton with qApp as a parent)
         * \param service The KService describing the plugin to return
         */
        AnnotationPlugin* getPluginInstance(const KSharedPtr<KService>& service);
    };

} // End namespace Nepomuk

#endif
